import java.awt.*;
import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import cryptix.math.BigInteger;
import cryptix.security.rsa.SecretKey;
import cryptix.security.SPEED;

class cryptoChatConnection extends Frame implements Runnable, consts {
  // Fr die Initialisierung der Verbindung braucht
  // der Client die folgenden Variabeln.
  String name;
  String serv;
  int port;
  String file;
  // Fr die Oberflche ...
  TextArea output = new TextArea( "", 12, 60 );
  TextArea input = new TextArea( "", 1, 60 );
  Label status = new Label( "Status: " );
  Panel pan = new Panel();
  GridBagLayout grid = new GridBagLayout();
  Button connButton = new Button( "Neue Verbindung..." );
  Button quitButton = new Button( "Verbindung beenden" );
  Button statButton = new Button( "Status" );
  // Schlielich die Variabeln fr die Netzwerkverbindung.
  Socket sock = null;
  DataInputStream inS = null;
  DataOutputStream outS = null;
  Thread read = null;
  SPEED speed;
  boolean isActive = false;

  // Wird u.a. aufgerufen, wenn wir das Fenster vergrern
  // oder verkleinern.
  public void paint( Graphics g ) {
    pan.resize( size().width - 5, pan.size().height );
    input.resize( size().width - 10, input.size().height );
    output.resize( size().width - 10, output.size().height );
  }

  cryptoChatConnection( String title ) {
    super( title );
    cryptoChatClient.activeConnections++;
    Panel buttonPanel = new Panel();

    buttonPanel.setLayout( new GridLayout( 1, 2 ) );
    add( "Center", buttonPanel );
    buttonPanel.add( connButton );
    buttonPanel.add( quitButton );
    buttonPanel.add( statButton );

    add( "West", pan );
    pan.setLayout( grid );
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.WEST;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.gridx = 1;
    c.gridheight = 1;
    c.gridy = 1;
    pan.add( buttonPanel );
    grid.setConstraints( buttonPanel, c );
    
    c.gridy = 2;
    c.weighty = 1.0;
    c.gridheight = 1;
    c.fill = GridBagConstraints.BOTH;
    pan.add( output );
    grid.setConstraints( output, c );
    output.setEditable( false );

    c.weighty = 0.0;
    c.gridy = 3;
    c.gridheight = 1;
    c.fill = GridBagConstraints.HORIZONTAL;
    pan.add( input );
    grid.setConstraints( input, c );

    c.gridy = 4;
    pan.add( status );
    grid.setConstraints( status, c );

    pack();
    int i =  (int) (java.lang.Math.random() * 200);
    reshape( 100 + i, 100 + i, 450, 450 );
    show();
    paint( getGraphics() );

    status( "Initialisierung" );

    // Malt den Initialisierungsdialog auf den Bildschirm
    // und blockiert, bis die Eingaben gemacht wurden.
    new initDialog( this );

    // Mit diesen Daten wird versucht eine Netzwerkverbindung
    // herzustellen.
    try {
      sock = new Socket( serv, port );
      outS = new DataOutputStream( sock.getOutputStream() );
      inS = new DataInputStream( sock.getInputStream() );
      if( sock == null ) 
	throw ( new Exception( "Keine Netzwerkverbindung!" ) ) ;
      // SecretKey einlesen.
      status( "Lese Schlssel..." );
      Properties skProp = new Properties();
      
      File fh = new File( file );
      FileInputStream skIS = new FileInputStream( fh );
      skProp.load( skIS );
      BigInteger n = new BigInteger( (String) skProp.getProperty("n") );
      BigInteger e = new BigInteger( (String) skProp.getProperty("e") );
      BigInteger d = new BigInteger( (String) skProp.getProperty("d") );
      BigInteger p = new BigInteger( (String) skProp.getProperty("p") );
      BigInteger q = new BigInteger( (String) skProp.getProperty("q") );
      BigInteger u = new BigInteger( (String) skProp.getProperty("u") );
      // SecretKey erzeugen.
      SecretKey sk = new SecretKey( n, e, d, p, q, u );
      status( "Sende ffentlichen Schlssel..." );
      message m;
      write( new message( NAME, name ) );
      write( new message( PK_E, e.toString() ) );
      write( new message( PK_N, n.toString() ) );
      m = message.parseMessage( inS.readLine() );
      if( m.code == PRIV_KEY ) {
	status( "Entschlssele geheimen Sitzungsschlssel." );
	byte[] speedKey = sk.decrypt( new BigInteger( m.data ) ).toByteArray();
	speed = new SPEED( speedKey, 32, 128 );
	if( inS != null ) {
	  read = new Thread( this );
	  read.start();
	  setTitle( name + "@" + serv );
	}
      } else {
	throw( new Exception( "Server hat keinen Schlssel gesendet!" ) );
      }
      isActive = true;
      status( "Bereit" );
    } catch( Exception e ) {
      output.appendText( "\n" + e + "\n" );
      output.appendText( "Kann Verbindung nicht aufbauen!" );
      input.setEditable( false );
      status( "Inaktiv" );
      setTitle( "Inaktiv" );
    }
  }

  // Ein eigener Thread lauscht permanent, ob Daten ber
  // das Netz kommen, entschlsselt sie und stellt sie dar.
  public void run() {
    while( true ) {
      try {
	message m = message.parseMessage( inS.readLine() );
	if( m.code == 200 ) {
	  int arraylength = (int) (m.data.length()/2/speed.blockLength()) *
	    speed.blockLength();
	  byte[] cipher = new byte[arraylength];
	  byte[] plain  = new byte[arraylength];
	  cipher = new BigInteger( m.data ).toByteArray();
	  for( int c = 0; c < arraylength; c += speed.blockLength() )
	    speed.decrypt( cipher, c, plain, c );
	  output.appendText( new String( plain, 0 ) + "\n" );
	} else {
	  output.appendText( m.data + "\n" );
	}
      } catch ( Exception e ) {
	output.appendText( "\n" + e );
	output.appendText( "\n *** Verbindung abgebrochen ***\n" );
	status( "Inaktiv" );
	setTitle( "Inaktiv" );
	input.setEditable( false );
	if( isActive )
	  read.stop();
      }
    }
  }

  // Wenn eine Schaltflche bettigt wurde ...
  public boolean action( Event evt, Object obj  ) {
    if( evt.target == quitButton ) {
      if( isActive )
	write( new message( QUIT, "" ) );
      cryptoChatClient.activeConnections--;
      dispose();
      return( true );
    } else if( evt.target == connButton ) {
      new cryptoChatConnection( "Neue Verbindung" );
      return( true );
    } else if( evt.target == statButton ) {
      write( new message( STATUS, "" ) );
      return( true );
    }
    return( false );
  }

  // Wenn im input-Feld Return gedrckt wurde,
  // wird der Inhalt versendet und gelscht.
  public boolean keyDown( Event evt, int key  ) {
    if( evt.target == input && key == '\n' )
      if( write( new message( name + ": " + input.getText() ) ) ) {
	input.setText( "" );
	return( true );
      }
    return( false );
  }

  boolean write( message m ) {
    if( m.code == DATA ) {
      int arraylength = 
	(int) ((m.data.length() - 1) / speed.blockLength() + 1) *
	speed.blockLength();
      byte[] cipher = new byte[arraylength];
      byte[] plain  = new byte[arraylength];
      m.data.getBytes( 0, m.data.length(), plain, 0 );
      for( int c = m.data.length(); c < arraylength; c++ )
	plain[c] = ' ';
      status( "Verschlsselung luft " );
      for( int c = 0; c < m.data.length(); c += speed.blockLength() ) {
	speed.encrypt( plain, c, cipher, c );
	status( status.getText() + "." );
	paint( getGraphics() );
      }
      String cipherString = new BigInteger( cipher ).toString();
      while( cipherString.length() < 2 * arraylength ) {
	cipherString = "0" + cipherString;
      }
      m.data = cipherString;
    }
    try {
      outS.writeBytes( m.code + m.data + "\n" );
      status( "Bereit" );
      return( true );
    } catch ( IOException ioe ) {
	output.appendText( "\n *** Verbindung abgebrochen ***\n" );
	input.setEditable( false );
	status( "Inaktiv" );
	setTitle( "Inaktiv" );
	if( read.isAlive() )
	  read.stop();
    }
    return( false );
  }

  void status( String message ) {
    status.setText( message );
    paint( getGraphics() );
  }
}

class cryptoChatClient {
  static String name = "";
  static String serv = "";
  static int port = 5555;
  static String file = "";
  static int activeConnections = 0;
  static final String usage = "java cryptoChatClient " +
  "[-s server] [-p port] [-n name] [-f datei]\n" +
  "  -s Server, zu dem eine Verbindung aufgebaut werden soll" +
  " (Default: keiner)\n" +
  "  -p TCP-Portnummer (Default: 5555)\n" +
  "  -n Name, der beim Chaten verwendet wird (Default: keiner)\n" +
  "  -f Datei, die einen RSA-SecretKey enthlt (Default: keiner)\n";

  public static void main( String arg[] ) {
    try {
      for( int i = 0; 2 * i < arg.length; i++ ) {
	if( arg[2 * i].indexOf( "-s" ) == 0 ) {
	  serv = arg[2 * i + 1];
	  System.out.println( "Server: " + serv );
	} else if( arg[2 * i].indexOf( "-p" ) == 0 ) {
	  port = Integer.parseInt( arg[2 * i + 1], 10 );
	  System.out.println( "Port: " + port );
	} else if( arg[2 * i].indexOf( "-f" ) == 0 ) {
	  file = arg[2 * i + 1];
	  System.out.println( "Datei: " + file );
	} else if( arg[2 * i].indexOf( "-n" ) == 0 ) {
	  name = arg[2 * i + 1];
	  System.out.println( "Name: " + name );
	} else {
	  System.out.println( usage );
	  System.exit( 1 );
	}
      }
    } catch ( NumberFormatException nfe ) {
      System.out.println( usage );
      System.exit( 1 );
    }
    
    new cryptoChatConnection( "Neue Verbindung" );
    while( activeConnections > 0 )
      try {      
	Thread.sleep( 100 );
      } catch ( InterruptedException ie ) {
      }
      System.exit( 0 );
  }
}

class initDialog extends Dialog {
  cryptoChatConnection conn = null;
  Button okButton = new Button( "Verbinden" );
  Button cancelButton = new Button( "Abbrechen" );

  TextField name = new TextField( cryptoChatClient.name );
  TextField serv = new TextField( cryptoChatClient.serv );
  TextField port = new TextField( new Integer( cryptoChatClient.port ).toString() );
  TextField file = new TextField( cryptoChatClient.file );

  initDialog( cryptoChatConnection f ) {
    super( f, "Verbindungsaufbau", true );
    conn = f;
    setLayout( new GridLayout( 5, 2 ) );
    add( new Label( "Name:", Label.RIGHT ) );
    add( name );
    add( new Label( "Server:", Label.RIGHT ) );
    add( serv );
    add( new Label( "Port:", Label.RIGHT ) );
    add( port );
    add( new Label( "SecretKey-Datei:", Label.RIGHT ) );
    add( file );
    add( okButton );
    add( cancelButton );
    pack();
    reshape( 100, 100, 220, 170 );
    show();
  }

  public boolean action( Event evt, Object obj ) {
    if( evt.target == okButton ) {
      conn.name = name.getText();
      conn.serv = serv.getText();
      try {
	conn.port = Integer.parseInt( port.getText() );
      } catch ( NumberFormatException nfe ) {
	port.setText( new Integer( cryptoChatClient.port ).toString() );
	return( true );
      }
      conn.file = file.getText();
      dispose();
      return( true );
    } else if( evt.target == cancelButton ) {
      conn.dispose();
      dispose();
      cryptoChatClient.activeConnections--;
      return( true );
    }
    return( false );
  }
}
