import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;

import cryptix.security.rsa.PublicKey;
import cryptix.security.SPEED;
import cryptix.math.BigInteger;

class connection extends Thread implements consts {
  boolean isActive = false;
  boolean isInitialised = false;
  static int active = 0;
  Socket sock = null;
  DataInputStream inS = null;
  DataOutputStream outS = null;
  InetAddress peer = null;
  Date start;
  String hostname ="";
  String name = "";
  String pk_e = "";
  String pk_n = "";
  PublicKey pk;

  void connection() {
  }

  void init( Socket sock ) {
    try {
      this.sock = sock;
      outS = new DataOutputStream( sock.getOutputStream() );
      inS = new DataInputStream( sock.getInputStream() );
      peer = sock.getInetAddress();
      hostname = peer.getHostName();
      start = new Date();
      active++;
      isActive = true;
      if( debug )
	System.out.println( "Client connection initiated." );
    } catch ( IOException ioe ) {
      System.out.println( "Cant init client connection." );
    }
  }

  void sendStatus() {
    write( new message( INFO, "Server Info:" ) );
    for( int i = 0; i < cryptoChatServer.maxconnections; i++ )
      if( cryptoChatServer.client[i] != null &&
	  cryptoChatServer.client[i].isActive &&
	  cryptoChatServer.client[i].isInitialised ) {
	write( new message( INFO,  cryptoChatServer.client[i].info() ) );
      }
  }

  String info() {
    return( name + "@" + peer.toString() +
	    " (" + start.toLocaleString() + ")" );
  }
    

  void deinit() {
    try {
      inS.close();
      outS.close();
      sock.close();
    } catch  ( IOException ioe ) {
      if ( debug )
	System.out.println( "connection.deinit(): " + ioe );
    }
    isActive = false;
    isInitialised = false;
    active--;
    inS = null;
    outS = null;
    sock = null;
    if ( debug )
      System.out.println( "Connection terminated" );
  }

  private synchronized void initPk() {
    if( !pk_n.equals( "" ) && 
	!pk_e.equals( "" ) && 
	!name.equals( "" ) ) {
      pk = new PublicKey( new BigInteger( pk_n ),
			  new BigInteger( pk_e ) );
      
      for( int i = 0; i < cryptoChatServer.maxconnections; i++ )
	if( cryptoChatServer.client[i] != null &&
	    cryptoChatServer.client[i].isActive &&
	    cryptoChatServer.client[i].isInitialised ) {
	  if ( debug )
	    System.out.println( "Wrote to " + i + "." );
	  cryptoChatServer.client[i].write( new message( NEW_CONN, "New connection: " + name + "@" + peer.toString() ) );
	}
      isInitialised = true;
      write( new message( PRIV_KEY, pk.encrypt( new BigInteger( cryptoChatServer.speedKey ) ).toString() ) );
      sendStatus();
    }
  }

  public void run() {
    while( isActive ) {
      message message = read();
      switch ( message.code ) {
      case NAME :
	if( !isInitialised ) {
	  name = message.data;
	  initPk();
	}
	break;
      case PK_N :
	if( !isInitialised ) {
	  pk_n = message.data;
	  initPk();
	}
	break;
      case PK_E :
	if( !isInitialised ) {
	  pk_e = message.data;
	  initPk();
	}
	break;
      case STATUS :
	sendStatus();
	break;
      case QUIT : 
	deinit();
	message = new message( INFO, "Bye from " + info() );
      case DATA :
	for( int i = 0; i < cryptoChatServer.maxconnections; i++ )
	  if( cryptoChatServer.client[i] != null &&
	      cryptoChatServer.client[i].isActive &&
	      cryptoChatServer.client[i].isInitialised ) {
	    cryptoChatServer.client[i].write( message );
	  }
	break;
      default:
	if( debug )
	  System.out.println( "Unknown code." );
	deinit();
	break;
      }
    }
    stop();
  }

  // connection.write() ist synchronized, damit sich
  // unterschiedliche Verbindungen auf dem Bildschirm der
  // Clients nicht ins Gehege kommen.
  synchronized void write( message message ) {
    try {
      outS.writeBytes( message.code + message.data + "\n" );
    } catch ( IOException ioe ) {
      deinit();
      if ( debug )
	System.out.println( "connection.write(): " + ioe );
    }
  }

  // connection.read() blockiert, bis etwas vom Client ankommt.
  message read() {
    message ret = new message( "" );
    try {
      ret = message.parseMessage( inS.readLine() );
    } catch  ( Exception e ) {
      if ( debug )
	System.out.println( "connection.read(): " + e );
      deinit();
    }
    if( debug )
      System.out.println( ret.code + ": " + ret.data );
    return( ret );
  }
}

class cryptoChatServer implements consts {
  static final int MAXCONNECTIONS = 5;
  static int listenPort = DEFAULT_PORT;
  static int maxconnections = MAXCONNECTIONS;
  static String usage = "usage: cryptoChatServer [-p port] [-c maxconnections]";
  static connection client[];
  static Listener listener;
  static boolean done = false;
  static byte[] speedKey = new byte[22];

  public static void main( String arg[] ) {
    try {
      for( int i = 0; 2 * i < arg.length; i++ ) {
	if( arg[2 * i].indexOf( "-c" ) == 0 ) {
	  maxconnections = Integer.parseInt( arg[2 * i + 1], 10 );
	  System.out.println( "Connections set to " + maxconnections );
	} else if( arg[2 * i].indexOf( "-p" ) == 0 ) {
	  listenPort = Integer.parseInt( arg[2 * i + 1], 10 );
	  System.out.println( "Port set to " + listenPort );
	} else {
	  System.out.println( usage );
	  System.exit( 1 );
	}
      }
    } catch ( NumberFormatException nfe ) {
      System.out.println( usage );
      System.exit( 1 );
    }
    Random r = new Random( new Date().getTime() );
    for( int i = 0; i < 22; i++ )
      speedKey[i] = (byte) ( r.nextInt() / 256 );
    client = new connection[maxconnections];
    listener = new Listener( listenPort );
    listener.start();
    while ( !done ) {
     try {
       Thread.sleep(50);
     } catch ( InterruptedException ie ) {
     }  
    }
  }
}

class Listener extends Thread implements consts {
  int listenPort;

  public Listener( int listenPort ) {
    this.listenPort = listenPort;
  }

  public void run() {
    ServerSocket listenTo;
    boolean done = false;

    while( true ) {
      try {
	listenTo = new ServerSocket( listenPort );
	//sleep( 20 );
	Socket clientSock = listenTo.accept();
	if ( debug )
	  System.out.println( "Processing connection request." );
	for( int i = 0; i < cryptoChatServer.maxconnections; i++ ) {
 	  if( cryptoChatServer.client[i] == null ||
	      !cryptoChatServer.client[i].isActive ) {
	    cryptoChatServer.client[i] = new connection();
	    cryptoChatServer.client[i].init( clientSock );
	    cryptoChatServer.client[i].start();

	    done = true;
	    if ( debug )
	      System.out.println( "New connection object initialised." );
	    break;
	  }
	}
	if( !done ) {
	  DataOutputStream outS =
	    new DataOutputStream( clientSock.getOutputStream() );
	  outS.writeBytes( "Server is busy!\n" );
	  if ( debug )
	    System.out.println( "Connection refused." );
	  clientSock.close();
	}
	listenTo.close();
	done = false;
      } catch ( Exception e ) {
	if( debug )
	  System.out.println( "listener.run(): " + e );
      }
    }
  }
}
