// Applet zur Verschlsselung von vom Benutzer eingegebenen
// Daten zur Kreditkartenabrechnung. Benutzung der Cryptix-
// Java-Klassenbibliothek in der Version 2.2
//
// (c) Hajo Passon, 1997 <hp@abacus.de>

// Aus der Cryptix-Library werden nur die Dinge importiert, die
// wir auch wirklich brauchen.
import cryptix.security.rsa.PublicKey;
import cryptix.math.MathError;
import cryptix.math.BigInteger;
import cryptix.math.BigNum;

import java.io.*;
import java.net.*;
import java.awt.*;
import java.applet.*;

public class rsaApplet extends Applet {
  private boolean debug = true;

  // Der Socket und die In-/Output-Streams mssen
  // innerhalb der Class zur Verfgung stehen aber
  // nicht auerhalb!
  private Socket mySock;
  private DataOutputStream send;
  private DataInputStream receive;

  // Ebenso der Public Key.
  private PublicKey key;

  private TextField ccname = new TextField( 30 );
  private Choice ccemit = new Choice();
  private TextField ccnum = new TextField( 16 );
  private TextField ccexpire = new TextField( 30 );
  private Button ccreset  = new Button( "Zurcksetzen" );
  private Button ccsubmit = new Button( "Absenden" );
  private Frame myFrame = new Frame();

  public void init() {
    if ( debug ) System.out.print( "Initialisiere ... " );
    
    // Parameter lesen und den Public Key konstruieren.
    try {
      key = new PublicKey( new BigInteger( getParameter( "n" ) ),
			   new BigInteger( getParameter( "e" ) ) );
    } catch ( NumberFormatException exc ) {
    }

    // Nun zum Aussehen.
    // Zunchst die Eintrge im Listenfeld fr
    // die Kreditkartengesellschaft.
    ccemit.addItem( "Visa" );
    ccemit.addItem( "Euro" );
    ccemit.addItem( "American Express" );

    // Die Kreditkartennummer wird - hnlich wie 
    // ein Password - nicht angezeigt.
    ccnum.setEchoCharacter( '*' );

    // Anordnen der einzelnen Elemente in einem GridLayout.
    setLayout( new GridLayout( 5, 2, 20, 20 ) );
    add( new Label( "Ihr Name:" ) );
    add( ccname );
    add( new Label( "Kreditkartengesellschaft:" ) );
    add( ccemit );
    add( new Label( "Gltig bis:" ) );
    add( ccexpire );
    add( new Label( "Kreditkartennummer:" ) );
    add( ccnum );
    add( ccreset );
    add( ccsubmit );
    if ( debug ) System.out.print( "fertig!\n" );
  }

  public boolean action( Event evt, Object obj )
  {
    String transmit = "";
    String secret = "";
    byte[] part;
    byte[] lpart;
    int i = 0;
    int j = 0;
    // Nur zu Debug-Zwecken:
    String answer;

    if ( evt.target == ccsubmit ){
      secret    = "RSA-PublicKey {\n\te = " +
	key.e() + ";\n\tn = " + key.n() + ";\n}\n";
      transmit  = "ccname="   + ccname.getText() + "\n";
      transmit += "ccemit="   + ccemit.getSelectedItem() + "\n";
      transmit += "ccexpire=" + ccexpire.getText() + "\n";
      transmit += "ccnum="    + ccnum.getText() + "\n";

      // Da einerseits weder bekannt ist, wie lang der Modulus
      // des public key, noch wie lang die zu verschlsselnde
      // Mitteilung ist, andererseits aber RSA ein Blockcipher-
      // Algorithmus ist, mu die Mitteilung in Zahlen
      // zerlegt werden, die mit Sicherheit kleiner als
      // der Modulus sind.
      // Die folgenden zwei Zeilen dimensionieren die byte arrays
      // fr einen ausreichend kleinen Block (part) und den letzten,
      // krzeren Block (lpart).
      part  = new byte[key.bitLength() / 8 - 1];
      lpart = new byte[transmit.length() % ( key.bitLength() / 8 - 1 )];

      secret += "-------BEGIN RSA-ENCRYPTED MESSAGE---------\n";
      while ( ( i + key.bitLength() / 8 - 1 ) < transmit.length() ) {
	for ( j = 0; j < key.bitLength() / 8 - 1 ; j++ ){
	  part[j] = (byte) transmit.charAt( i + j );
	}
	secret += key.encrypt( new BigInteger( part ) ).toString() + "\n";
	i += key.bitLength() / 8 - 1;
      }
      for ( j = 0; j < lpart.length; j++ ){
	lpart[j] = (byte) transmit.charAt( i + j );
      }
      secret += key.encrypt( new BigInteger( lpart ) ).toString() + "\n";
      secret += "--------END RSA-ENCRYPTED MESSAGE----------\n";

      // Der gesamte Netzwerkzugriff kann IOExceptions
      // auslsen. Deshalb wird er als ganzes in eine
      // try-catch-Struktur verlegt.
      try {
	// Zunchst werden der Socket und die Ein-/Ausgabekanle
	// initialisiert. Wir sprechen - der Einfachheit halber -
	// einen Prozess an, der von Hause aus ein gut dokumentiertes
	// und funktionierendes Protokoll spricht: Den smtp-Server
	// auf TCP-Port 25.
	myFrame.pack();
	myFrame.resize( 0, 0 );
	myFrame.move( 320, 240 );
	mySock  = new Socket( getDocumentBase().getHost(), 25 );
	receive = new DataInputStream( mySock.getInputStream() );
	send    = new DataOutputStream( mySock.getOutputStream() );

	// Zunchst die Verbindungsmeldung vom Server abholen;
	// Erst mal "Hallo" sagen. Leitet eine smtp-Kommunikation ein.
	// Wer ist der Absender? (Nicht sehr elegant - funktioniert aber.)
	// Wer ist der Empfnger? Das steht im Applet-Tag als Parameter.
	// Jetzt kommt der Inhalt der Mail.
	// Wir schicken unsere Nachricht und anschlieend eine
	// Zeile, die nur einen Punkt enthlt.
	// Verabschiedung und schlieen des Socket.
	Exception e = new Exception( "Problem mit Netzwerkzugriff.\n" );
	if ( !smtpSend( "" ) ) throw( e );
	if ( !smtpSend( "HELO localhost\n" ) ) throw( e ); 
	if ( !smtpSend( "MAIL FROM: nobody\n" ) ) throw( e );
	if ( !smtpSend( "RCPT TO: " + getParameter( "to" ) + "\n" ) ) throw( e );
	if ( !smtpSend( "DATA\n" ) ) throw( e );
	if ( !smtpSend( secret + "\n.\n" ) ) throw( e );
	smtpSend( "QUIT\n" );
	new alert( myFrame, "Datenbertragung war erfolgreich!" );
      } catch ( Exception e ) {
        if ( debug ) System.out.print( "Problem mit Netzwerkzugriff.\n" );
        new alert( myFrame, "Datenbertragung war *nicht* erfolgreich!" );
      }
      if ( debug ) System.out.print( transmit );
      if ( debug ) System.out.print( secret );
    }
    if ( evt.target == ccreset ) {
      ccname.setText( "" );
      ccexpire.setText( "" );
      ccnum.setText( "" );
    }
    return true;
  }

  private boolean smtpSend( String question ) 
  {
    String answer;
    int i = 0;
    try {
      if ( debug ) System.out.print( question );
      send.writeBytes( question );
      answer = receive.readLine();
      if ( debug ) System.out.println( answer );

      // Der Server mu auf alle mglichen Anfragen mit einer
      // Zeile der Form: "220 xxx.abacus.de ESMTP 
      // Sendmail 8.8.5/8.8.5; Tue, 3 Jun 1997 17:39:10 +0200"
      // antworten. Die Return-Codes am Zeilenanfang sollten
      // immer Serverim 200er Bereich liegen. 300er sind Warnungen,
      // 400er und 500er Fehler.
      if ( answer.charAt( 0 ) == '2' || answer.charAt( 0 ) == '3' )
	return true;
    } catch ( IOException ioe ) {
      if ( debug ) System.out.println( "Problem bei Netzzugriff." );
      return false;
    }
    return false;
  }

  // Nur der Vollstndigkeit halber: Methoden, die jedes Applet
  // kennen mu, die in diesem Fall aber leer sind, da nur
  // Standard-AWT-Komponenten auf den Bildschirm gemalt werden.
  public void start() {}

  public void stop() {}

  public void destroy() {}

  public void run() {}

  public void paint(Graphics d) {}
}
